#!/bin/bash
#  download pdb file from PDB and finds gap hetero, metals and chains in the pdb file 
#  and printout relevant info 
# 
# syntax: 
#   ./find_gap.bash [-f] pdbcode
#    "pdbcode" is the alphanumeric code of the PDB file
#    if -f option is used; no wget from PDB ; file with pdbcode=xxxx 
#    [ i.e. pdbxxxx.ent] is assumed to be local  
#

local=0
while getopts ":fl" opt; do 
    case $opt in 
	f) 
	    local=1; 
	    ;;
	l) 
	    local=1; 
	    ;;
	\?) 
	    echo "invalid option"
	    ;;
    esac
done
code=${!#}

# wget the file if -f is not given 
if [ $local != 1 ] ; then 
    wget ftp://ftp.wwpdb.org/pub/pdb/data/structures/all/pdb/pdb${code}.ent.gz; 
    gzip -d pdb${code}.ent.gz ; mv   pdb${code}.ent ${code}.pdb
else
    echo "HERE"
    echo "QUIIIIIII<br>"
    mv $code . 
    extf=`echo $code | awk '{substr($1,length($1)-3),4)}'`
    echo "HERE" $extf "<br>"
    if [ $extf != ".pdb" ] ; then
	echo  "Not a pdb file"
	exit
    else
      code=`echo $code | awk '{substr($1,1,length($1)-4)}'` 
      echo  "newcode " $code $extf  
      exit
    fi
fi

# scanning for general info 
echo ""  > $code.OUT
echo " == GENERAL SECTION  " >> $code.OUT
( printf "    NRESIDUE "; grep "ATOM " ${code}.pdb | grep " CA " | wc -l  ) >> $code.OUT
awk '/TITLE/{print "   ", $0}' ${code}.pdb >> $code.OUT
awk '/DOI/{print "   ", $0}' ${code}.pdb >> $code.OUT
 
# scan for chains info
echo "" >> $code.OUT
echo " == CHAIN SECTION " >> $code.OUT
awk '{\
    if($1=="ATOM") {\
	{\
	    c=substr($0,22,1); \
	    n[c]+=1;\
	} \
    }\
}\
END {\
    if(n["A"]!=0) print "    chain A", n["A"];  \
    if(n["B"]!=0) print "    chain B", n["B"];  \
    if(n["C"]!=0) print "    chain C", n["C"];  \
    if(n["D"]!=0) print "    chain D", n["D"];  \
    if(n["E"]!=0) print "    chain E", n["A"];  \
    if(n["F"]!=0) print "    chain F", n["B"];  \
    if(n["G"]!=0) print "    chain G", n["C"];  \
    if(n["H"]!=0) print "    chain H", n["D"];  \
}' ${code}.pdb >> $code.OUT



# scanning for gaps 
echo "" >> $code.OUT
echo " == GAP SECTION  " >> $code.OUT

grep -e "^ATOM " ${code}.pdb | grep " CA " | awk \
'{\
    ires=substr($0,23,4); name=substr($0,18,3); chain=substr($0,22,1); dr=ires-ires_old;\
    if(dr>1) {\
       printf "%5d%5s%2s%5d%5s%2s  GAP%4d\n",  ires_old,name_old,chain_old,ires,name,chain,dr\
    };\
    ires_old=substr($0,23,4);\
    name_old=substr($0,18,3);\
    chain_old=substr($0,22,1)\
}' >> $code.OUT 

# search for hetero and metals 
echo "" >> $code.OUT
echo " == HETERO ATOMS SECTION "  >> $code.OUT

awk ' BEGIN{nres_old=-100} \
{\
    if($1=="HETATM"){ \
	i++;\
	het=substr($0,18,3); \
	cha=substr($0,22,1); \
	nres=substr($0,23,5); \
	if(nres!=nres_old) {\
	    if(nres_old!=-100) { \
		j++;\
		hetero[j]=het_old
		netero[j]=i; \
                chain[j]=chain_old
	    }\
	    nres_old=nres;\
	    het_old=het;\
	    chain_old=cha;\
	    i=0\
	}\
    }\
}\
END {\
    nw=0;
    for(k=1; k<=j; k++) {\
	if(hetero[k]!="HOH") {\
		printf "    %3s%2s natom=%3d\n",  hetero[k],chain[k],netero[k]\
	} else {\
         nw++
        }\
    }\
    if(nw>0) { printf "    %6s n=%3d\n",  "WATER",nw}\
}' ${code}.pdb >> $code.OUT

echo "<pre>"
cat $code.OUT
echo "</pre>"

# create a new html page for action fixpdb.php 
echo "<html>" > $code.html
echo "<form action="fixpdb.php" method="post" enctype="multipart/form-data">"  >>  $code.html
echo "<pre>" >>  $code.html
cat $code.OUT >>  $code.html 
echo "</pre>" >>  $code.html
echo "<form action="fixpdb.php" method="post" enctype="multipart/form-data">"
-
echo "<center>"  >>$code.html
echo " PDB   <select name="pdbcode">" >>$code.html
echo "    <option value="${code}">$code </option>" >>$code.html
echo "</select>" >>$code.html

echo " CHAIN? <select name="chain">" >>$code.html
echo "    <option value="A">A </option>" >>$code.html
echo "    <option value="B">B </option>" >>$code.html
echo "    <option value="C">C </option>" >>$code.html
echo "    <option value="D">D </option>" >>$code.html
echo "</select>" >>$code.html

echo " Fix gaps? <select name="gaps">" >>$code.html
echo "    <option value="yes">yes </option>" >>$code.html
echo "    <option value="no">no </option>" >>$code.html
echo "</select>" >>$code.html

echo " Ligand? <input type="text" name="ligand " />"   >>$code.html
 >>$code.html
echo "</center>"  >>$code.html
echo '<p><input type="submit" name="Submit" style="width:200px; height:50px; font-size:20px; border: red; border-style:solid; border-width:3px" value="submit" /></p>' >>$code.html
echo "</center>"  >>$code.html

echo "</html>" >>$code.html
echo "<html>"
echo "<b> Make your choices for chain and ligands and go to <a href="$code.html"> THIS PAGE </a> "
echo "<html>"

